@@ -39,7 +39,7 @@ X" option to 11, "Spacing Y" to 11 and "Major grid line every" to 1.
 
 Before your icon is ready to be used in Sugar, it needs to be Sugarized.  This
 converts the colors to SVG entities, which allows Sugar to change the colors
-of the icon.  To do that, just run the `sugar-iconfiy`__ script.  Usually, it
+of the icon.  To do that, just run the `sugar-iconify`__ script.  Usually, it
 "just works" like::
 
     python path/to/sugar-iconify.py -o icon.svg
@@ -87,14 +87,18 @@ In this example, the badge will be centered at 97.0% on the X axis,
 and 85.0% on the Y axis.
 '''
 
+import six
 import re
 import math
 import logging
 import os
-from ConfigParser import ConfigParser
+
+from six.moves.configparser import ConfigParser
 
 import gi
 gi.require_version('Rsvg', '2.0')
+gi.require_version('Gtk', '3.0')
+from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 from gi.repository import Gdk
@@ -112,7 +116,7 @@ _BADGE_SIZE = 0.45
 class _SVGLoader(object):
 
     def __init__(self):
-        self._cache = LRU(50)
+        self._cache = LRU(100)
 
     def load(self, file_name, entities, cache):
         if file_name in self._cache:
@@ -125,8 +129,8 @@ class _SVGLoader(object):
             if cache:
                 self._cache[file_name] = icon
 
-        for entity, value in entities.items():
-            if isinstance(value, basestring):
+        for entity, value in list(entities.items()):
+            if isinstance(value, six.string_types):
                 xml = '<!ENTITY %s "%s">' % (entity, value)
                 icon = re.sub('<!ENTITY %s .*>' % entity, xml, icon)
             else:
@@ -155,7 +159,7 @@ class _BadgeInfo(object):
 
 class _IconBuffer(object):
 
-    _surface_cache = LRU(50)
+    _surface_cache = LRU(100)
     _loader = _SVGLoader()
 
     def __init__(self):
@@ -204,7 +208,9 @@ class _IconBuffer(object):
         elif info.get_filename():
             # try read from the .icon file
             icon_filename = info.get_filename().replace('.svg', '.icon')
-            if os.path.exists(icon_filename):
+            if icon_filename != info.get_filename() and \
+                            os.path.exists(icon_filename):
+
                 try:
                     with open(icon_filename) as config_file:
                         cp = ConfigParser()
@@ -374,7 +380,7 @@ class _IconBuffer(object):
                         icon_width = pixbuf.get_width()
                         icon_height = pixbuf.get_height()
                         break
-                    except GObject.GError:
+                    except GLib.GError:
                         pass
 
         if icon_width is None:
@@ -466,7 +472,6 @@ class Icon(Gtk.Image):
 
     __gtype_name__ = 'SugarIcon'
 
-    # FIXME: deprecate icon_size
     _MENU_SIZES = (Gtk.IconSize.MENU, Gtk.IconSize.DND,
                    Gtk.IconSize.SMALL_TOOLBAR, Gtk.IconSize.BUTTON)
 
@@ -479,7 +484,6 @@ class Icon(Gtk.Image):
         self._alpha = 1.0
         self._scale = 1.0
 
-        # FIXME: deprecate icon_size
         if 'icon_size' in kwargs:
             logging.warning("icon_size is deprecated. Use pixel_size instead.")
 
@@ -492,7 +496,7 @@ class Icon(Gtk.Image):
         self._file = file_name
         self._buffer.file_name = file_name
 
-    file = GObject.property(type=object, setter=set_file, getter=get_file)
+    file = GObject.Property(type=object, setter=set_file, getter=get_file)
 
     def get_pixbuf(self):
         '''
@@ -515,7 +519,7 @@ class Icon(Gtk.Image):
         '''
         self._buffer.pixbuf = pixbuf
 
-    pixbuf = GObject.property(type=object, setter=set_pixbuf,
+    pixbuf = GObject.Property(type=object, setter=set_pixbuf,
                               getter=get_pixbuf)
     '''
     icon.props.pixbuf -> see :any:`get_pixbuf` and :any:`set_pixbuf`
@@ -528,7 +532,6 @@ class Icon(Gtk.Image):
         if self._buffer.file_name != self.props.file:
             self._buffer.file_name = self.props.file
 
-        # FIXME: deprecate icon_size
         pixel_size = None
         if self.props.pixel_size == -1:
             if self.props.icon_size in self._MENU_SIZES:
@@ -545,7 +548,7 @@ class Icon(Gtk.Image):
             self._buffer.height = height
 
     def _icon_size_changed_cb(self, image, pspec):
-        self._buffer.icon_size = self.props.icon_size
+        self._buffer.icon_size = self.props.pixel_size
 
     def _icon_name_changed_cb(self, image, pspec):
         self._buffer.icon_name = self.props.icon_name
@@ -624,7 +627,7 @@ class Icon(Gtk.Image):
             self._buffer.xo_color = value
             self.queue_draw()
 
-    xo_color = GObject.property(
+    xo_color = GObject.Property(
         type=object, getter=None, setter=set_xo_color)
     '''
     icon.props.xo_color -> see :any:`set_xo_color`, note there is no getter
@@ -650,7 +653,7 @@ class Icon(Gtk.Image):
         '''
         return self._buffer.fill_color
 
-    fill_color = GObject.property(
+    fill_color = GObject.Property(
         type=object, getter=get_fill_color, setter=set_fill_color)
     '''
     icon.props.fill_color -> see :any:`get_fill_color`
@@ -677,7 +680,7 @@ class Icon(Gtk.Image):
         '''
         return self._buffer.stroke_color
 
-    stroke_color = GObject.property(
+    stroke_color = GObject.Property(
         type=object, getter=get_stroke_color, setter=set_stroke_color)
     '''
     icon.props.stroke_color -> see :any:`get_stroke_color`
@@ -704,7 +707,7 @@ class Icon(Gtk.Image):
         '''
         return self._buffer.badge_name
 
-    badge_name = GObject.property(
+    badge_name = GObject.Property(
         type=str, getter=get_badge_name, setter=set_badge_name)
     '''
     icon.props.badge_name -> see :any:`get_badge_name`
@@ -730,7 +733,7 @@ class Icon(Gtk.Image):
             self._alpha = value
             self.queue_draw()
 
-    alpha = GObject.property(
+    alpha = GObject.Property(
         type=float, setter=set_alpha)
     '''
     icon.props.alpha -> see :any:`set_alpha`, note no getter
@@ -749,7 +752,7 @@ class Icon(Gtk.Image):
             self._scale = value
             self.queue_draw()
 
-    scale = GObject.property(
+    scale = GObject.Property(
         type=float, setter=set_scale)
     '''
     icon.props.scale -> see :any:`set_scale`, note no getter
@@ -801,7 +804,7 @@ class EventIcon(Gtk.EventBox):
         # for example, after a touch palette invocation
         self.connect_after('button-release-event',
                            self.__button_release_event_cb)
-        for key, value in kwargs.iteritems():
+        for key, value in six.iteritems(kwargs):
             self.set_property(key, value)
 
         from sugar3.graphics.palette import CursorInvoker
@@ -858,7 +861,7 @@ class EventIcon(Gtk.EventBox):
     def get_file_name(self):
         return self._buffer.file_name
 
-    file_name = GObject.property(
+    file_name = GObject.Property(
         type=object, getter=get_file_name, setter=set_file_name)
 
     def set_icon_name(self, value):
@@ -869,7 +872,7 @@ class EventIcon(Gtk.EventBox):
     def get_icon_name(self):
         return self._buffer.icon_name
 
-    icon_name = GObject.property(
+    icon_name = GObject.Property(
         type=object, getter=get_icon_name, setter=set_icon_name)
 
     def set_xo_color(self, value):
@@ -877,7 +880,7 @@ class EventIcon(Gtk.EventBox):
             self._buffer.xo_color = value
             self.queue_draw()
 
-    xo_color = GObject.property(
+    xo_color = GObject.Property(
         type=object, getter=None, setter=set_xo_color)
 
     def set_fill_color(self, value):
@@ -888,7 +891,7 @@ class EventIcon(Gtk.EventBox):
     def get_fill_color(self):
         return self._buffer.fill_color
 
-    fill_color = GObject.property(
+    fill_color = GObject.Property(
         type=object, getter=get_fill_color, setter=set_fill_color)
 
     def set_stroke_color(self, value):
@@ -899,7 +902,7 @@ class EventIcon(Gtk.EventBox):
     def get_stroke_color(self):
         return self._buffer.stroke_color
 
-    stroke_color = GObject.property(
+    stroke_color = GObject.Property(
         type=object, getter=get_stroke_color, setter=set_stroke_color)
 
     def set_background_color(self, value):
@@ -919,7 +922,7 @@ class EventIcon(Gtk.EventBox):
         '''
         return self._buffer.background_color
 
-    background_color = GObject.property(
+    background_color = GObject.Property(
         type=object, getter=get_background_color, setter=set_background_color)
     '''
     event_icon.props.get_background_color -> see :any:`set_background_color`
@@ -935,7 +938,7 @@ class EventIcon(Gtk.EventBox):
     def get_size(self):
         return self._buffer.width
 
-    pixel_size = GObject.property(
+    pixel_size = GObject.Property(
         type=object, getter=get_size, setter=set_size)
 
     def set_scale(self, value):
@@ -946,7 +949,7 @@ class EventIcon(Gtk.EventBox):
     def get_scale(self):
         return self._buffer.scale
 
-    scale = GObject.property(
+    scale = GObject.Property(
         type=float, getter=get_scale, setter=set_scale)
 
     def set_alpha(self, alpha):
@@ -954,7 +957,7 @@ class EventIcon(Gtk.EventBox):
             self._alpha = alpha
             self.queue_draw()
 
-    alpha = GObject.property(
+    alpha = GObject.Property(
         type=float, setter=set_alpha)
 
     def set_cache(self, value):
@@ -974,7 +977,7 @@ class EventIcon(Gtk.EventBox):
         '''
         return self._buffer.cache
 
-    cache = GObject.property(
+    cache = GObject.Property(
         type=bool, default=False, getter=get_cache, setter=set_cache)
     '''
     event_icon.props.cache -> see :any:`set_cache` and :any:`get_cache`
@@ -988,7 +991,7 @@ class EventIcon(Gtk.EventBox):
     def get_badge_name(self):
         return self._buffer.badge_name
 
-    badge_name = GObject.property(
+    badge_name = GObject.Property(
         type=object, getter=get_badge_name, setter=set_badge_name)
 
     def create_palette(self):
@@ -1032,7 +1035,7 @@ class EventIcon(Gtk.EventBox):
         '''
         self._palette_invoker.palette = palette
 
-    palette = GObject.property(
+    palette = GObject.Property(
         type=object, setter=set_palette, getter=get_palette)
     '''
     event_icon.props.palette -> see :any:`get_palette` and :any:`set_palette`
@@ -1045,7 +1048,7 @@ class EventIcon(Gtk.EventBox):
         self._palette_invoker.detach()
         self._palette_invoker = palette_invoker
 
-    palette_invoker = GObject.property(
+    palette_invoker = GObject.Property(
         type=object, setter=set_palette_invoker, getter=get_palette_invoker)
 
     def set_tooltip(self, text):
@@ -1148,6 +1151,8 @@ class CanvasIcon(EventIcon):
 
     def __palette_popdown_cb(self, palette):
         self.unset_state_flags(Gtk.StateFlags.PRELIGHT)
+
+
 if hasattr(CanvasIcon, 'set_css_name'):
     CanvasIcon.set_css_name('canvasicon')
 
@@ -1199,13 +1204,13 @@ class CellRendererIcon(Gtk.CellRenderer):
         if self._buffer.file_name != value:
             self._buffer.file_name = value
 
-    file_name = GObject.property(type=str, setter=set_file_name)
+    file_name = GObject.Property(type=str, setter=set_file_name)
 
     def set_icon_name(self, value):
         if self._buffer.icon_name != value:
             self._buffer.icon_name = value
 
-    icon_name = GObject.property(type=str, setter=set_icon_name)
+    icon_name = GObject.Property(type=str, setter=set_icon_name)
 
     def get_xo_color(self):
         return self._xo_color
@@ -1213,40 +1218,40 @@ class CellRendererIcon(Gtk.CellRenderer):
     def set_xo_color(self, value):
         self._xo_color = value
 
-    xo_color = GObject.property(type=object,
+    xo_color = GObject.Property(type=object,
                                 getter=get_xo_color, setter=set_xo_color)
 
     def set_fill_color(self, value):
         if self._fill_color != value:
             self._fill_color = value
 
-    fill_color = GObject.property(type=object, setter=set_fill_color)
+    fill_color = GObject.Property(type=object, setter=set_fill_color)
 
     def set_stroke_color(self, value):
         if self._stroke_color != value:
             self._stroke_color = value
 
-    stroke_color = GObject.property(type=object, setter=set_stroke_color)
+    stroke_color = GObject.Property(type=object, setter=set_stroke_color)
 
     def set_prelit_fill_color(self, value):
         if self._prelit_fill_color != value:
             self._prelit_fill_color = value
 
-    prelit_fill_color = GObject.property(type=object,
+    prelit_fill_color = GObject.Property(type=object,
                                          setter=set_prelit_fill_color)
 
     def set_prelit_stroke_color(self, value):
         if self._prelit_stroke_color != value:
             self._prelit_stroke_color = value
 
-    prelit_stroke_color = GObject.property(type=object,
+    prelit_stroke_color = GObject.Property(type=object,
                                            setter=set_prelit_stroke_color)
 
     def set_background_color(self, value):
         if self._buffer.background_color != value:
             self._buffer.background_color = value
 
-    background_color = GObject.property(type=object,
+    background_color = GObject.Property(type=object,
                                         setter=set_background_color)
 
     def set_size(self, value):
@@ -1256,7 +1261,7 @@ class CellRendererIcon(Gtk.CellRenderer):
 
             self._cached_offsets = None
 
-    size = GObject.property(type=object, setter=set_size)
+    size = GObject.Property(type=object, setter=set_size)
 
     def do_get_size(self, widget, cell_area, x_offset=None, y_offset=None,
                     width=None, height=None):
@@ -1443,6 +1448,6 @@ def get_surface(**kwargs):
         cairo surface or None if image was not found
     '''
     icon = _IconBuffer()
-    for key, value in kwargs.items():
+    for key, value in list(kwargs.items()):
         icon.__setattr__(key, value)
     return icon.get_surface()
